@@ -3,11 +3,14 @@
 package appconfig
 
 import (
+	"encoding/base64"
 	"fmt"
 	"net/url"
+	"os"
 	"reflect"
 
 	"github.com/superfly/flyctl/api"
+	"github.com/superfly/flyctl/internal/machine"
 	"github.com/superfly/flyctl/scanner"
 	"golang.org/x/exp/slices"
 )
@@ -37,6 +40,7 @@ type Config struct {
 	PrimaryRegion  string        `toml:"primary_region,omitempty" json:"primary_region,omitempty"`
 	KillSignal     *string       `toml:"kill_signal,omitempty" json:"kill_signal,omitempty"`
 	KillTimeout    *api.Duration `toml:"kill_timeout,omitempty" json:"kill_timeout,omitempty"`
+	SwapSizeMB     *int          `toml:"swap_size_mb,omitempty" json:"swap_size_mb,omitempty"`
 	ConsoleCommand string        `toml:"console_command,omitempty" json:"console_command,omitempty"`
 
 	// Sections that are typically short and benefit from being on top
@@ -51,6 +55,10 @@ type Config struct {
 	HTTPService *HTTPService              `toml:"http_service,omitempty" json:"http_service,omitempty"`
 	Services    []Service                 `toml:"services,omitempty" json:"services,omitempty"`
 	Checks      map[string]*ToplevelCheck `toml:"checks,omitempty" json:"checks,omitempty"`
+	Files       []File                    `toml:"files,omitempty" json:"files,omitempty"`
+
+	// MergedFiles is a list of files that have been merged from the app config and flags.
+	MergedFiles []*api.File `toml:"-" json:"-"`
 
 	// Others, less important.
 	Statics []Static            `toml:"statics,omitempty" json:"statics,omitempty"`
@@ -79,6 +87,35 @@ type Deploy struct {
 	Strategy       string `toml:"strategy,omitempty" json:"strategy,omitempty"`
 }
 
+type File struct {
+	GuestPath  string   `toml:"guest_path" json:"guest_path,omitempty" validate:"required"`
+	LocalPath  string   `toml:"local_path" json:"local_path,omitempty"`
+	SecretName string   `toml:"secret_name" json:"secret_name,omitempty"`
+	RawValue   string   `toml:"raw_value" json:"raw_value,omitempty"`
+	Processes  []string `json:"processes,omitempty" toml:"processes,omitempty"`
+}
+
+func (f File) toMachineFile() (*api.File, error) {
+	file := &api.File{
+		GuestPath: f.GuestPath,
+	}
+	switch {
+	case f.LocalPath != "":
+		content, err := os.ReadFile(f.LocalPath)
+		if err != nil {
+			return nil, fmt.Errorf("could not read file %s: %w", f.LocalPath, err)
+		}
+		rawValue := base64.StdEncoding.EncodeToString(content)
+		file.RawValue = &rawValue
+	case f.SecretName != "":
+		file.SecretName = &f.SecretName
+	case f.RawValue != "":
+		encodedValue := base64.StdEncoding.EncodeToString([]byte(f.RawValue))
+		file.RawValue = &encodedValue
+	}
+	return file, nil
+}
+
 type Static struct {
 	GuestPath string `toml:"guest_path" json:"guest_path,omitempty" validate:"required"`
 	UrlPrefix string `toml:"url_prefix" json:"url_prefix,omitempty" validate:"required"`
@@ -253,3 +290,28 @@ func (cfg *Config) URL() *url.URL {
 func (cfg *Config) PlatformVersion() string {
 	return cfg.platformVersion
 }
+
+// MergeFiles merges the provided files with the files in the config wherein the provided files
+// take precedence.
+func (cfg *Config) MergeFiles(files []*api.File) error {
+	// First convert the Config files to Machine files.
+	cfgFiles := make([]*api.File, 0, len(cfg.Files))
+	for _, f := range cfg.Files {
+		machineFile, err := f.toMachineFile()
+		if err != nil {
+			return err
+		}
+		cfgFiles = append(cfgFiles, machineFile)
+	}
+
+	// Merge the config files with the provided files.
+	mConfig := &api.MachineConfig{
+		Files: files,
+	}
+	machine.MergeFiles(mConfig, files)
+
+	// Persist the merged files back to the config to be used later for deploying.
+	cfg.MergedFiles = mConfig.Files
+
+	return nil
+}
