@@ -13,6 +13,7 @@ import (
 	"github.com/superfly/flyctl/internal/command"
 	"github.com/superfly/flyctl/internal/config"
 	"github.com/superfly/flyctl/internal/flag"
+	"github.com/superfly/flyctl/internal/flag/flagnames"
 	"github.com/superfly/flyctl/internal/format"
 	"github.com/superfly/flyctl/internal/render"
 )
@@ -87,7 +88,7 @@ func runList(ctx context.Context) (err error) {
 
 func getOrg(ctx context.Context) (*api.Organization, error) {
 	client := client.FromContext(ctx).API()
-	orgName := flag.GetString(ctx, flag.OrgName)
+	orgName := flag.GetString(ctx, flagnames.Org)
 
 	if orgName == "" {
 		return nil, nil
